/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */


package org.experlog.db;

import org.experlog.openeas.api.DataObject;
import org.experlog.util.Client;

import java.sql.CallableStatement;
import java.sql.SQLException;
import java.sql.Connection;
import java.util.Hashtable;

/**
 * This class makes some delegation works on a CallableStatement object.
 * We store an hashtable of parameters to get results
 */
public class ESCallable implements DataObject {
    // the delegate callable statement
    private CallableStatement callable;

    // all the parameters are stored
    // key: String, the name of the parameter
    // value: String, the value of the parameter
    private Hashtable outparam = null;

    // the name of the datasource on which the callable statement is built
    private String datasource = null;

    private Connection con_ = null;

    private Client session_ = null;
    /**
     * create a new object with a CallableStatement object and initialize a
     * list of parameters
     * @param stat  a CallableStatement object
     */
    public ESCallable(CallableStatement stat, String adatasource, Connection con, Client session) {
        callable = stat;
        outparam = new Hashtable();
	con_ = con;
	session_ = session;
	datasource = adatasource;
    }

    /**
     * registers a parameter with the position in the call string (@see the
     * CallableStatement API) and the int representation of the parameter type.
     * @param pos   the position of the parameter in the call string
     * @param type  the type of the parameter (@see java.sql.Types)
     */
    public void registerOutParameter(int pos, int type) {
        try {
        callable.registerOutParameter(pos, type);
        } catch (SQLException sql) {
            sql.printStackTrace();
            System.err.println("impossible to register an out parameter on" +
                    "the current callable statement");
        }

        // register the parameter in our internal list, "null" is the current
        // value
        outparam.put(new Integer(pos).toString(), "null");
    }

    /**
     * returns the list of the parameters
     * @return an Hashtable object which contains (key,value) of (parameter,
     * parameter_value)
     */
    public Hashtable getParams() {
        return outparam;
    }

    /**
     * returns the underlying callable statement object
     * @return  the current and unique CallableStatement object
     */
    public CallableStatement getCallable() {
        return callable;
    }

    /**
     * returns the value of the a given out parameter
     * @param name  the parameter name
     * @return  the parameter value
     * @throws SQLException
     */
    public String get(String name) throws SQLException{
        return (String)(outparam.get(name));
    }

    /**
     * returns the name of the datasource which has been used to built the
     * current callable statement
     * @return  the string name of the datasource
     */
    public String getDatasource() {
        return datasource;
    }

    /**
     * closes all objects, this methods is called at the end of life of the
     * current object.
     * @throws SQLException
     */
    public void close() throws SQLException {
        datasource = null;
        outparam.clear();
        callable.close();
	session_.closeConnection(con_, datasource);
    }

    public Connection getConnection() {
	return con_;
    }
}
