/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.util;

import java.sql.SQLException;
import java.util.StringTokenizer;

import org.experlog.openeas.api.DataSet;
import org.experlog.openeas.api.DataObject;

public class ESEnumDataSet implements DataSet {

	StringTokenizer stok_ = null;
	int position_ = 0;
	int low_ = 0;
	int high_ = -1;

	public ESEnumDataSet(String enu, String sep) throws SQLException {
		StringBuffer b = new StringBuffer();
		StringTokenizer st = new StringTokenizer(enu, sep, true); //delims=tokens
    boolean istoken = true;
		while (st.hasMoreTokens()) {

			String ct = st.nextToken();
      boolean isdelim = isDelim(ct, sep);

      if(isdelim) {
        if(!istoken) b.append("00void00" + sep); // 00void00 stands for "empty"
      } else {
			  int pos = ct.indexOf("->");
			  if (pos > 0 && ct.length() > pos + 2) {
				  String v1 = ct.substring(0, pos);
				  String v2 = ct.substring(pos + 2);
				  try {
					  int i1 = Integer.parseInt(v1);
					  int i2 = Integer.parseInt(v2);
					  if(i1 > i2) {
					    for (int i = i1; i >= i2; i--) {
                          b.append(Integer.toString(i) + sep);
                        }
					  } else {
					    for (int i = i1; i <= i2; i++) {
					      b.append(Integer.toString(i) + sep);
					    }
					  }
				  } catch (NumberFormatException nfx) {
					  b.append(ct + sep);
				  }
			  } else {
				  b.append(ct + sep);
			  }
      }

      istoken = !isdelim;
		}
		stok_ = new StringTokenizer(b.toString(), sep);
	}

  private boolean isDelim(String tok, String sep) {
    return (sep.indexOf(tok) >= 0);
  }

	public boolean next() throws SQLException {
		boolean ret = dataAvailable();
		if(ret) position_ ++;
		return ret;
	}

	public boolean dataAvailable() throws SQLException {
		return ((high_ < low_ || position_ <= high_) && stok_.hasMoreTokens());
	}

	public DataObject getTuple() throws SQLException {
		if (!stok_.hasMoreTokens())
     throw new SQLException("No more data available");
		ESSimpleDataTuple dt = new ESSimpleDataTuple();
    String val = stok_.nextToken();
    if(val.equals("00void00")) val = ""; // 00void00 stands for "empty"
		dt.put("value", val);
		return dt;
	}

	public void open() throws SQLException {
	}

	public void close() throws SQLException {
	}

	public boolean isTransient() {
		return true;
	}

	public void setBoundaries(int low, int high) throws SQLException {
		if(low<=0) low = 1;
		low_ = low-1;
		high_ = high-1;
		while(position_ < low_ && next()) stok_.nextToken(); // Skip to low index
	}

}

