/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.util;

import java.util.Collections;
import java.util.Comparator;
import java.util.Vector;
import java.sql.SQLException;

import org.experlog.openeas.api.DataSet;
import org.experlog.openeas.api.DataObject;
import org.experlog.openeas.api.VectorDataSet;

class DataObjectComparator implements Comparator {
  int asc_ = 1;
  String col_;

  public DataObjectComparator(String col, boolean asc) {
    col_ = col;
    asc_ = (asc ? 1 : -1);
  }

  public int compare(Object o1, Object o2) {
    String s1, s2;
    try {
      s1 = (o1 == null ? "" : ((DataObject)o1).get(col_));
    } catch(SQLException e) {
      s1 = "";
    }
    try {
      s2 = (o2 == null ? "" : ((DataObject)o2).get(col_));
    } catch(SQLException e) {
      s2 = "";
    }
    return asc_ * s1.compareTo(s2);
  }
}

public class ESMaterializeDataSet implements DataSet {

  VectorDataSet dset_ = null;
  Vector materialized_ = new Vector();
  boolean transient_ = false;

  public ESMaterializeDataSet(DataSet ds) throws SQLException {
    if(ds != null) { // consider the dataset empty
      ds.open();
      while(ds.next()) {
        materialized_.addElement(ds.getTuple());
      }
      ds.close();
    }
    dset_ = new VectorDataSet(materialized_);
    dset_.setTransient(transient_);
  }

  public void orderBy(String col, boolean asc) {
    Collections.sort(materialized_, new DataObjectComparator(col, asc));
  }

  public boolean next() throws SQLException {
    return dset_.next();
  }

  public boolean dataAvailable() throws SQLException {
    return dset_.dataAvailable();
  }

  public DataObject getTuple() throws SQLException {
    return dset_.getTuple();
  }

  public void open() throws SQLException {
    dset_.open();
  }

  public void close() throws SQLException {
    dset_.close();
  }

  public boolean isTransient() { return transient_; }

  public void setBoundaries(int low, int high) throws SQLException {
    dset_.setBoundaries(low, high);
  }
}

